<!-- DO NOT EDIT: this document is automatically generated by script/gendoc/gendoc.go -->

Configuration
=============

You can configure Fireworq by providing environment variables on
starting a daemon (for both docker-composed and manually-set-up
instances) or by specifying command line arguments (for a manual
setup).  Command line arguments precede the values of environment
variables.

The following variables/arguments are available.  Some of them are
applicable only to a [manual setup][section-manual-setup].

- [Common Variables/Arguments][section-config-common]
  - [`FIREWORQ_ACCESS_LOG`, `--access-log`](#env-access-log)
  - [`FIREWORQ_ACCESS_LOG_TAG`, `--access-log-tag`](#env-access-log-tag)
  - [`FIREWORQ_DISPATCH_IDLE_CONN_TIMEOUT`, `--dispatch-idle-conn-timeout`](#env-dispatch-idle-conn-timeout)
  - [`FIREWORQ_DISPATCH_KEEP_ALIVE`, `--dispatch-keep-alive`](#env-dispatch-keep-alive)
  - [`FIREWORQ_DISPATCH_MAX_CONNS_PER_HOST`, `--dispatch-max-conns-per-host`](#env-dispatch-max-conns-per-host)
  - [`FIREWORQ_DISPATCH_USER_AGENT`, `--dispatch-user-agent`](#env-dispatch-user-agent)
  - [`FIREWORQ_ERROR_LOG`, `--error-log`](#env-error-log)
  - [`FIREWORQ_ERROR_LOG_LEVEL`, `--error-log-level`](#env-error-log-level)
  - [`FIREWORQ_KEEP_ALIVE`, `--keep-alive`](#env-keep-alive)
  - [`FIREWORQ_PID`, `--pid`](#env-pid)
  - [`FIREWORQ_QUEUE_DEFAULT`, `--queue-default`](#env-queue-default)
  - [`FIREWORQ_QUEUE_DEFAULT_MAX_WORKERS`, `--queue-default-max-workers`](#env-queue-default-max-workers)
  - [`FIREWORQ_QUEUE_DEFAULT_POLLING_INTERVAL`, `--queue-default-polling-interval`](#env-queue-default-polling-interval)
  - [`FIREWORQ_QUEUE_LOG`, `--queue-log`](#env-queue-log)
  - [`FIREWORQ_QUEUE_LOG_LEVEL`, `--queue-log-level`](#env-queue-log-level)
  - [`FIREWORQ_QUEUE_LOG_TAG`, `--queue-log-tag`](#env-queue-log-tag)
- [Variables/Arguments only Applicable to Manual Setup][section-config-manual-setup]
  - [`FIREWORQ_BIND`, `--bind`](#env-bind)
  - [`FIREWORQ_CONFIG_REFRESH_INTERVAL`, `--config-refresh-interval`](#env-config-refresh-interval)
  - [`FIREWORQ_DRIVER`, `--driver`](#env-driver)
  - [`FIREWORQ_MYSQL_DSN`, `--mysql-dsn`](#env-mysql-dsn)
  - [`FIREWORQ_QUEUE_MYSQL_DSN`, `--queue-mysql-dsn`](#env-queue-mysql-dsn)
  - [`FIREWORQ_REPOSITORY_MYSQL_DSN`, `--repository-mysql-dsn`](#env-repository-mysql-dsn)
  - [`FIREWORQ_SHUTDOWN_TIMEOUT`, `--shutdown-timeout`](#env-shutdown-timeout)
- [Variables only Applicable to a Docker-composed Instance][section-config-docker]
  - [`FIREWORQ_PORT`](#env-port)

## <a name="config-common">Common Variables/Arguments</a>
### <a name="env-access-log">`FIREWORQ_ACCESS_LOG`, `--access-log`</a>

Specifies a file where API access log is written to.  It defaults to standard output.

Each line in the file is a JSON string corresponds to a single log item.

### <a name="env-access-log-tag">`FIREWORQ_ACCESS_LOG_TAG`, `--access-log-tag`</a>
Default: `fireworq.access`

Specifies the value of `tag` field in a access log item.

### <a name="env-dispatch-idle-conn-timeout">`FIREWORQ_DISPATCH_IDLE_CONN_TIMEOUT`, `--dispatch-idle-conn-timeout`</a>
Default: `0`

Specifies the maximum amount of time of an idle (keep-alive) connection will remain idle before closing itself. If zero, an idle connections will not be closed. 

### <a name="env-dispatch-keep-alive">`FIREWORQ_DISPATCH_KEEP_ALIVE`, `--dispatch-keep-alive`</a>

Specifies whether a connection to a worker should be reused.  This overrides [the default keep-alive setting](#env-keep-alive).

### <a name="env-dispatch-max-conns-per-host">`FIREWORQ_DISPATCH_MAX_CONNS_PER_HOST`, `--dispatch-max-conns-per-host`</a>
Default: `10`

Specifies maximum idle connections to keep per-host. This value works only when [connections of the dispatcher are reused](#env-dispatch-keep-alive).

### <a name="env-dispatch-user-agent">`FIREWORQ_DISPATCH_USER_AGENT`, `--dispatch-user-agent`</a>

Specifies the value of `User-Agent` header field used for an HTTP request to a worker.  The default value is <code>Fireworq/<var>version</var></code>.

### <a name="env-error-log">`FIREWORQ_ERROR_LOG`, `--error-log`</a>

Specifies a file where error logs are written to.  It defaults to standard error output.

If this value is specified, each line in the file is a JSON string corresponds to a single log item.  Otherwise, each line of the output is a prettified log item.

### <a name="env-error-log-level">`FIREWORQ_ERROR_LOG_LEVEL`, `--error-log-level`</a>

Specifies a log level of the access log.  The level is either a name or a numeric value.  The following table describes the meaning of the value.

|Value|Name   |
|-----|-------|
|`0`    |`debug`  |
|`1`    |`info`   |
|`2`    |`warn`   |
|`3`    |`error`  |
|`4`    |`fatal`  |

If none of these values is specified, the level is determined by `DEBUG` environment variable.  If `DEBUG` has a non-empty value, then the level is `debug`.  Otherwise, the level is `info`.

### <a name="env-keep-alive">`FIREWORQ_KEEP_ALIVE`, `--keep-alive`</a>
Default: `false`

Specifies whether connections should be reused.

### <a name="env-pid">`FIREWORQ_PID`, `--pid`</a>

Specifies a file where PID is written to.

### <a name="env-queue-default">`FIREWORQ_QUEUE_DEFAULT`, `--queue-default`</a>

Specifies the name of a default queue.  A job whose `category` is not defined via the [routing API][api-put-routing] will be delivered to this queue.  If no default queue name is specified, pushing a job with an unknown category will fail for a [manual setup][section-manual-setup].  A docker-composed instance uses `default` as a default value.

If you already have a queue with the specified name in the job queue database, that one is used.  Or otherwise a new queue is created automatically.

### <a name="env-queue-default-max-workers">`FIREWORQ_QUEUE_DEFAULT_MAX_WORKERS`, `--queue-default-max-workers`</a>
Default: `20`

Specifies the default maximum number of jobs that are processed simultaneously in a queue, used when `max_workers` in the [queue API][api-put-queue] is omitted.

### <a name="env-queue-default-polling-interval">`FIREWORQ_QUEUE_DEFAULT_POLLING_INTERVAL`, `--queue-default-polling-interval`</a>
Default: `200`

Specifies the default interval, in milliseconds, at which Fireworq checks the arrival of new jobs, used when `polling_interval` in the [queue API][api-put-queue] is omitted.

### <a name="env-queue-log">`FIREWORQ_QUEUE_LOG`, `--queue-log`</a>

Specifies a file where the job queue logs are written to.  It defaults to standard output. No other logs than the job queue logs are written to this file.

Each line in the file is a JSON string corresponds to a single log item.

### <a name="env-queue-log-level">`FIREWORQ_QUEUE_LOG_LEVEL`, `--queue-log-level`</a>

Specifies a log level of the job queue logs.  The level is either a name or a numeric value.  The following table describes the meaning of the value.

|Value|Name   |
|-----|-------|
|`0`    |`debug`  |
|`1`    |`info`   |
|`2`    |`warn`   |
|`3`    |`error`  |
|`4`    |`fatal`  |

If none of these values is specified, the level is determined by `DEBUG` environment variable.  If `DEBUG` has a non-empty value, then the level is `debug`.  Otherwise, the level is `info`.

### <a name="env-queue-log-tag">`FIREWORQ_QUEUE_LOG_TAG`, `--queue-log-tag`</a>
Default: `fireworq.queue`

Specifies the value of `tag` field in a job queue log item JSON.


## <a name="config-manual-setup">Variables/Arguments only Applicable to Manual Setup</a>
### <a name="env-bind">`FIREWORQ_BIND`, `--bind`</a>
Default: `127.0.0.1:8080`

Specifies the address and the port number of a daemon in a form <code><var>address</var>:<var>port</var></code>.

### <a name="env-config-refresh-interval">`FIREWORQ_CONFIG_REFRESH_INTERVAL`, `--config-refresh-interval`</a>
Default: `1000`

Specifies an interval, in milliseconds, at which a Fireworq daemon checks if configurations (such as queue definitions or routings) are changed by other daemons.

### <a name="env-driver">`FIREWORQ_DRIVER`, `--driver`</a>
Default: `mysql`

Specifies a driver for job queues and repositories.  The available values are `mysql` and `in-memory`.

Note that `in-memory` driver is not for production use.  It is intended to be used for just playing with Fireworq without a storage middleware or to show the upper bound of performance in a benchmark.

### <a name="env-mysql-dsn">`FIREWORQ_MYSQL_DSN`, `--mysql-dsn`</a>
Default: `tcp(localhost:3306)/fireworq`

Specifies a data source name for the job queue and the repository database in a form <code><var>user</var>:<var>password</var>@tcp(<var>mysql_host</var>:<var>mysql_port</var>)/<var>database</var>?<var>options</var></code>.  This is in effect only when [the driver](#env-driver) is `mysql` and is mandatory for that case.

### <a name="env-queue-mysql-dsn">`FIREWORQ_QUEUE_MYSQL_DSN`, `--queue-mysql-dsn`</a>

Specifies a data source name for the job queue database in a form <code><var>user</var>:<var>password</var>@tcp(<var>mysql_host</var>:<var>mysql_port</var>)/<var>database</var>?<var>options</var></code>.  This is in effect only when the [driver](#env-driver) is `mysql` and overrides [the default DSN](#env-mysql-dsn).  This should be used when you want to specify a DSN differs from [the repository DSN](#env-repository-mysql-dsn).

### <a name="env-repository-mysql-dsn">`FIREWORQ_REPOSITORY_MYSQL_DSN`, `--repository-mysql-dsn`</a>

Specifies a data source name for the repository database in a form <code><var>user</var>:<var>password</var>@tcp(<var>mysql_host</var>:<var>mysql_port</var>)/<var>database</var>?<var>options</var></code>.  This is in effect only when the [driver](#env-driver) is `mysql` and overrides [the default DSN](#env-mysql-dsn).  This should be used when you want to specify a DSN differs from [the queue DSN](#env-queue-mysql-dsn).

### <a name="env-shutdown-timeout">`FIREWORQ_SHUTDOWN_TIMEOUT`, `--shutdown-timeout`</a>
Default: `30`

Specifies a timeout, in seconds, which the daemon waits on [gracefully shutting down or restarting][section-graceful-restart].


## <a name="config-docker">Variables only Applicable to a Docker-composed Instance</a>

### <a name="env-port">`FIREWORQ_PORT`</a>

Default: `8080`

Specifies the port number of a daemon.

[section-config-common]: #config-common
[section-config-manual-setup]: #config-manual-setup
[section-config-docker]: #config-docker
[section-manual-setup]: ./production.md#manual-setup
[section-graceful-restart]: ./production.md#graceful-restart

[api-put-queue]: ./api.md#api-put-queue
[api-put-routing]: ./api.md#api-put-routing
